// ------------------------------------------------------------------------------
// PythagorasTree - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Shape.java
// The class representing a triangle with two squares on the top sides. 
// ------------------------------------------------------------------------------

class Shape
{
  int child1;
  int child2;
  float x1;
  float y1;
  float x2;
  float y2;
  float x3;
  float y3;
  float x4;
  float y4;
  float x5;
  float y5;
  float x6;
  float y6;
  float x7;
  float y7;
  boolean canAddChild;       // has it been ordered to respawn?
  boolean willAddChild;      // is it a leaf in the tree?

  Shape( float X1, float Y1, float X2, float Y2 )
  {
    canAddChild = false;
    willAddChild = true;
    assign(X1,Y1,X2,Y2);
    numc++;
  }

  void calculate()
  {
    // Calculates the various points
    // on the square and on the triangle
    
    float theta = PI/2-atan2(x2-x1, y2-y1);
    float r = sqrt(pow(x2-x1,2)+pow(y2-y1,2));

    float xt3 = r/2;
    float yt3 = cos(radians((mouseX-width/2)/2))*r/2;
    
    if ( yt3 < 0 ) { yt3 = 0; }

    x3 = x1 + cos(theta)*xt3 + sin(theta)*yt3;
    y3 = y1 + sin(theta)*xt3 - cos(theta)*yt3;
    
    float deltax = x2-x3;
    float deltay = y2-y3;

    x4 = x2 + deltay;
    y4 = y2 - deltax;
    x5 = x3 + deltay;
    y5 = y3 - deltax;
    
    deltax = x3-x1;
    deltay = y3-y1;

    x6 = x3 + deltay;
    y6 = y3 - deltax;
    x7 = x1 + deltay;
    y7 = y1 - deltax;
  }
  
  void recalculate()
  {
    calculate();
    
    // If you change the points on this shape
    // you will also need to update the points
    // on its children.
    if ( child1 > 0 )
    {
      shapes[child1].assign(x5,y5,x4,y4);
      shapes[child2].assign(x7,y7,x6,y6);
    }
  }

  void display()
  {
    //Draw the square and the triangle
    
    beginShape(POLYGON);
    vertex(x1,y1);
    vertex(x2,y2);
    vertex(x3,y3);
    endShape();
    
    beginShape(POLYGON);
    vertex(x3,y3);
    vertex(x2,y2);
    vertex(x4,y4);
    vertex(x5,y5);
    endShape();
    
    beginShape(POLYGON);
    vertex(x1,y1);
    vertex(x3,y3);
    vertex(x6,y6);
    vertex(x7,y7);
    endShape();
  }

  void assign( float X1, float Y1, float X2, float Y2 )
  {
    x1 = X1;
    y1 = Y1;
    x2 = X2;
    y2 = Y2;
  }
  
  void addChildren()
  {
    if ( canAddChild == true && willAddChild == true )
    {  
      child1 = numc;
      shapes[numc] = new Shape(x5, y5, x4, y4);
      child2 = numc;
      shapes[numc] = new Shape(x7, y7, x6, y6);
      
      canAddChild = false;
      willAddChild = false;
    }
  }
}
